#
#
#

import hashlib
import hmac
import logging
from collections import defaultdict
from time import gmtime, sleep, strftime

from requests import Session

from octodns import __VERSION__ as octodns_version
from octodns.provider import ProviderException
from octodns.provider.base import BaseProvider
from octodns.record import Record

__VERSION__ = '0.0.3'


class DnsMadeEasyClientException(ProviderException):
    pass


class DnsMadeEasyClientBadRequest(DnsMadeEasyClientException):
    def __init__(self, resp):
        errors = '\n  - '.join(resp.json()['error'])
        super().__init__(f'\n  - {errors}')


class DnsMadeEasyClientUnauthorized(DnsMadeEasyClientException):
    def __init__(self):
        super().__init__('Unauthorized')


class DnsMadeEasyClientNotFound(DnsMadeEasyClientException):
    def __init__(self):
        super().__init__('Not Found')


class DnsMadeEasyClient(object):
    PRODUCTION = 'https://api.dnsmadeeasy.com/V2.0/dns/managed'
    SANDBOX = 'https://api.sandbox.dnsmadeeasy.com/V2.0/dns/managed'

    def __init__(self, api_key, secret_key, sandbox=False, ratelimit_delay=0.0):
        self.api_key = api_key
        self.secret_key = secret_key
        self._base = self.SANDBOX if sandbox else self.PRODUCTION
        self.ratelimit_delay = ratelimit_delay
        self._sess = Session()
        self._sess.headers.update(
            {
                'x-dnsme-apiKey': self.api_key,
                'User-Agent': f'octodns/{octodns_version} octodns-dnsmadeasy/{__VERSION__}',
            }
        )
        self._domains = None

    def _current_time(self):
        return strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())

    def _hmac_hash(self, now):
        return hmac.new(
            self.secret_key.encode(), now.encode(), hashlib.sha1
        ).hexdigest()

    def _request(self, method, path, params=None, data=None):
        now = self._current_time()
        hmac_hash = self._hmac_hash(now)

        headers = {'x-dnsme-hmac': hmac_hash, 'x-dnsme-requestDate': now}

        url = f'{self._base}{path}'
        resp = self._sess.request(
            method, url, headers=headers, params=params, json=data
        )
        if resp.status_code == 400:
            raise DnsMadeEasyClientBadRequest(resp)
        if resp.status_code in [401, 403]:
            raise DnsMadeEasyClientUnauthorized()
        if resp.status_code == 404:
            raise DnsMadeEasyClientNotFound()
        resp.raise_for_status()
        sleep(self.ratelimit_delay)
        return resp

    @property
    def domains(self):
        if self._domains is None:
            zones = []

            # has pages in resp, do we need paging?
            resp = self._request('GET', '/').json()
            zones += resp['data']

            self._domains = {f'{z["name"]}.': z['id'] for z in zones}

        return self._domains

    def domain(self, name):
        path = f'/id/{name}'
        return self._request('GET', path).json()

    def domain_create(self, name):
        response = self._request('POST', '/', data={'name': name}).json()
        # Add our newly created domain to the cache
        self.domains[f'{name}.'] = response['id']

    def records(self, zone_name):
        zone_id = self.domains.get(zone_name, False)
        if not zone_id:
            return []
        path = f'/{zone_id}/records'
        ret = []

        # has pages in resp, do we need paging?
        resp = self._request('GET', path).json()
        ret += resp['data']

        for record in ret:
            # change ANAME records to ALIAS
            if record['type'] == 'ANAME':
                record['type'] = 'ALIAS'

            # change relative values to absolute
            value = record['value']
            if record['type'] in ['ALIAS', 'CNAME', 'MX', 'NS', 'SRV']:
                if value == '':
                    record['value'] = zone_name
                elif not value.endswith('.'):
                    record['value'] = f'{value}.{zone_name}'

        return ret

    def record_multi_delete(self, zone_name, record_ids):
        zone_id = self.domains.get(zone_name, False)
        path = f'/{zone_id}/records'
        self._request('DELETE', path, params={'ids': record_ids})

    def record_multi_create(self, zone_name, records):
        zone_id = self.domains.get(zone_name, False)
        path = f'/{zone_id}/records/createMulti'

        # Change ALIAS records to ANAME
        for record in records:
            if record['type'] == 'ALIAS':
                record['type'] = 'ANAME'
            record['gtdLocation'] = 'DEFAULT'
        self._request('POST', path, data=records)


class DnsMadeEasyProvider(BaseProvider):
    SUPPORTS_GEO = False
    SUPPORTS_DYNAMIC = False
    SUPPORTS_ROOT_NS = True
    SUPPORTS = set(
        (
            'A',
            'AAAA',
            'ALIAS',
            'CAA',
            'CNAME',
            'MX',
            'NS',
            'PTR',
            'SPF',
            'SRV',
            'TXT',
        )
    )

    def __init__(
        self,
        id,
        api_key,
        secret_key,
        sandbox=False,
        ratelimit_delay=0.0,
        *args,
        **kwargs,
    ):
        self.log = logging.getLogger(f'DnsMadeEasyProvider[{id}]')
        self.log.debug(
            '__init__: id=%s, api_key=***, secret_key=***, sandbox=%s',
            id,
            sandbox,
        )
        super().__init__(id, *args, **kwargs)
        self._client = DnsMadeEasyClient(
            api_key, secret_key, sandbox, ratelimit_delay
        )

        self._zone_records = {}

    def _data_for_multiple(self, _type, records):
        return {
            'ttl': records[0]['ttl'],
            'type': _type,
            'values': [r['value'] for r in records],
        }

    _data_for_A = _data_for_multiple
    _data_for_AAAA = _data_for_multiple
    _data_for_NS = _data_for_multiple

    def _data_for_CAA(self, _type, records):
        values = []
        for record in records:
            values.append(
                {
                    'flags': record['issuerCritical'],
                    'tag': record['caaType'],
                    'value': record['value'][1:-1],
                }
            )
        return {'ttl': records[0]['ttl'], 'type': _type, 'values': values}

    def _data_for_TXT(self, _type, records):
        values = [value['value'].replace(';', '\\;') for value in records]
        return {'ttl': records[0]['ttl'], 'type': _type, 'values': values}

    _data_for_SPF = _data_for_TXT

    def _data_for_MX(self, _type, records):
        values = []
        for record in records:
            values.append(
                {'preference': record['mxLevel'], 'exchange': record['value']}
            )
        return {'ttl': records[0]['ttl'], 'type': _type, 'values': values}

    def _data_for_single(self, _type, records):
        record = records[0]
        return {'ttl': record['ttl'], 'type': _type, 'value': record['value']}

    _data_for_CNAME = _data_for_single
    _data_for_PTR = _data_for_single
    _data_for_ALIAS = _data_for_single

    def _data_for_SRV(self, _type, records):
        values = []
        for record in records:
            values.append(
                {
                    'port': record['port'],
                    'priority': record['priority'],
                    'target': record['value'],
                    'weight': record['weight'],
                }
            )
        return {'type': _type, 'ttl': records[0]['ttl'], 'values': values}

    def zone_records(self, zone):
        if zone.name not in self._zone_records:
            self._zone_records[zone.name] = self._client.records(zone.name)

        return self._zone_records[zone.name]

    def populate(self, zone, target=False, lenient=False):
        self.log.debug(
            'populate: name=%s, target=%s, lenient=%s',
            zone.name,
            target,
            lenient,
        )

        values = defaultdict(lambda: defaultdict(list))
        for record in self.zone_records(zone):
            _type = record['type']
            if _type not in self.SUPPORTS:
                self.log.warning(
                    'populate: skipping unsupported %s record', _type
                )
                continue
            values[record['name']][record['type']].append(record)

        before = len(zone.records)
        for name, types in values.items():
            for _type, records in types.items():
                data_for = getattr(self, f'_data_for_{_type}')
                record = Record.new(
                    zone,
                    name,
                    data_for(_type, records),
                    source=self,
                    lenient=lenient,
                )
                zone.add_record(record, lenient=lenient)

        exists = zone.name in self._zone_records
        self.log.info(
            'populate:   found %s records, exists=%s',
            len(zone.records) - before,
            exists,
        )
        return exists

    def supports(self, record):
        # DNS Made Easy does not support empty/NULL SRV records
        #
        # Attempting to sync such a record would generate the following error
        #
        # octodns.provider.dnsmadeeasy.DnsMadeEasyClientBadRequest:
        #      - Record value may not be a standalone dot.
        #
        # Skip the record and continue
        if record._type == "SRV":
            if 'value' in record.data:
                targets = (record.data['value']['target'],)
            else:
                targets = [value['target'] for value in record.data['values']]

            if "." in targets:
                self.log.warning(
                    'supports: unsupported %s record with target (%s)',
                    record._type,
                    targets,
                )
                return False

        return super().supports(record)

    def _params_for_multiple(self, record):
        for value in record.values:
            yield {
                'value': value,
                'name': record.name,
                'ttl': record.ttl,
                'type': record._type,
            }

    _params_for_A = _params_for_multiple
    _params_for_AAAA = _params_for_multiple

    # An A record with this name must exist in this domain for
    # this NS record to be valid. Need to handle checking if
    # there is an A record before creating NS
    _params_for_NS = _params_for_multiple

    def _params_for_single(self, record):
        yield {
            'value': record.value,
            'name': record.name,
            'ttl': record.ttl,
            'type': record._type,
        }

    _params_for_CNAME = _params_for_single
    _params_for_PTR = _params_for_single
    _params_for_ALIAS = _params_for_single

    def _params_for_MX(self, record):
        for value in record.values:
            yield {
                'value': value.exchange,
                'name': record.name,
                'mxLevel': value.preference,
                'ttl': record.ttl,
                'type': record._type,
            }

    def _params_for_SRV(self, record):
        for value in record.values:
            yield {
                'value': value.target,
                'name': record.name,
                'port': value.port,
                'priority': value.priority,
                'ttl': record.ttl,
                'type': record._type,
                'weight': value.weight,
            }

    def _params_for_TXT(self, record):
        # DNSMadeEasy does not want values escaped
        for value in record.chunked_values:
            yield {
                'value': value.replace('\\;', ';'),
                'name': record.name,
                'ttl': record.ttl,
                'type': record._type,
            }

    _params_for_SPF = _params_for_TXT

    def _params_for_CAA(self, record):
        for value in record.values:
            yield {
                'value': value.value,
                'issuerCritical': value.flags,
                'name': record.name,
                'caaType': value.tag,
                'ttl': record.ttl,
                'type': record._type,
            }

    def _mod_Create(self, change):
        creations = []
        new = change.new
        params_for = getattr(self, f'_params_for_{new._type}')
        for params in params_for(new):
            creations.append(params)
        return new.zone, [], creations

    def _mod_Delete(self, change):
        deletions = []
        existing = change.existing
        zone = existing.zone
        for record in self.zone_records(zone):
            if (
                existing.name == record['name']
                and existing._type == record['type']
            ):
                deletions.append(record['id'])
        return zone, deletions, []

    def _mod_Update(self, change):
        _, deletions, _ = self._mod_Delete(change)
        zone, _, creations = self._mod_Create(change)
        return zone, deletions, creations

    def _apply(self, plan):
        desired = plan.desired
        changes = plan.changes
        self.log.debug(
            '_apply: zone=%s, len(changes)=%d', desired.name, len(changes)
        )

        domain_name = desired.name[:-1]
        try:
            self._client.domain(domain_name)
        except DnsMadeEasyClientNotFound:
            self.log.debug('_apply:   no matching zone, creating domain')
            self._client.domain_create(domain_name)

        zone_operations = {}

        # Optimise our changes into a single set of creates/delete operations for each zone
        for change in changes:
            class_name = change.__class__.__name__
            zone, mod_del, mod_create = getattr(self, f'_mod_{class_name}')(
                change
            )
            if zone.name in zone_operations:
                zone_operations[zone.name]['deletions'].extend(mod_del)
                zone_operations[zone.name]['creations'].extend(mod_create)
            else:
                zone_operations[zone.name] = {
                    'zone': zone,
                    'deletions': mod_del,
                    'creations': mod_create,
                }

        # Perform our operations
        for zone_name, operation in zone_operations.items():
            if operation['deletions']:
                self._client.record_multi_delete(
                    zone_name, operation['deletions']
                )
            if operation['creations']:
                self._client.record_multi_create(
                    zone_name, operation['creations']
                )

        # Clear out the cache if any
        self._zone_records.pop(desired.name, None)



#
#
#

import json
from os.path import dirname, join
from unittest import TestCase
from unittest.mock import Mock, call

from requests import HTTPError
from requests_mock import ANY
from requests_mock import mock as requests_mock

from octodns.provider.yaml import YamlProvider
from octodns.record import Record
from octodns.zone import Zone

from octodns_dnsmadeeasy import DnsMadeEasyClientNotFound, DnsMadeEasyProvider


class TestDnsMadeEasyProvider(TestCase):
    expected = Zone('unit.tests.', [])
    source = YamlProvider('test', join(dirname(__file__), 'config'))
    source.populate(expected)

    # Our test suite differs a bit, add our NS and remove the simple one
    expected.add_record(
        Record.new(
            expected,
            'under',
            {
                'ttl': 3600,
                'type': 'NS',
                'values': ['ns1.unit.tests.', 'ns2.unit.tests.'],
            },
        )
    )

    # Add some ALIAS records
    expected.add_record(
        Record.new(
            expected,
            '',
            {'ttl': 1800, 'type': 'ALIAS', 'value': 'aname.unit.tests.'},
        )
    )

    for record in list(expected.records):
        if record.name == 'sub' and record._type == 'NS':
            expected._remove_record(record)
            break

    def test_populate(self):
        provider = DnsMadeEasyProvider('test', 'api', 'secret')

        # not found
        with requests_mock() as mock:
            mock.get(ANY, status_code=404, text='{"error": ["Not Found"]}')

            with self.assertRaises(Exception) as ctx:
                zone = Zone('unit.tests.', [])
                provider.populate(zone)
            self.assertEqual('Not Found', str(ctx.exception))

        # Bad auth
        with requests_mock() as mock:
            mock.get(
                ANY, status_code=401, text='{"error": ["API key not found"]}'
            )

            with self.assertRaises(Exception) as ctx:
                zone = Zone('unit.tests.', [])
                provider.populate(zone)
            self.assertEqual('Unauthorized', str(ctx.exception))

        # Bad request
        with requests_mock() as mock:
            mock.get(
                ANY, status_code=400, text='{"error": ["Rate limit exceeded"]}'
            )

            with self.assertRaises(Exception) as ctx:
                zone = Zone('unit.tests.', [])
                provider.populate(zone)
            self.assertEqual('\n  - Rate limit exceeded', str(ctx.exception))

        # General error
        with requests_mock() as mock:
            mock.get(ANY, status_code=502, text='Things caught fire')

            with self.assertRaises(HTTPError) as ctx:
                zone = Zone('unit.tests.', [])
                provider.populate(zone)
            self.assertEqual(502, ctx.exception.response.status_code)

        # No diffs == no changes
        with requests_mock() as mock:
            base = 'https://api.dnsmadeeasy.com/V2.0/dns/managed'
            with open('tests/fixtures/dnsmadeeasy-domains.json') as fh:
                mock.get(f'{base}/', text=fh.read())
            with open('tests/fixtures/dnsmadeeasy-records.json') as fh:
                mock.get(f'{base}/123123/records', text=fh.read())

                zone = Zone('unit.tests.', [])
                provider.populate(zone)
                self.assertEqual(14, len(zone.records))
                changes = self.expected.changes(zone, provider)
                self.assertEqual(0, len(changes))

        # 2nd populate makes no network calls/all from cache
        again = Zone('unit.tests.', [])
        provider.populate(again)
        self.assertEqual(14, len(again.records))

        # bust the cache
        del provider._zone_records[zone.name]

    def test_populate_empty(self):
        provider = DnsMadeEasyProvider('test', 'api', 'secret')

        # Non-existent zone doesn't populate anything
        with requests_mock() as mock:
            with open('tests/fixtures/dnsmadeeasy-no-domains.json') as fh:
                mock.get(ANY, text=fh.read())

            zone = Zone('unit.tests.', [])
            provider.populate(zone)
            self.assertEqual(set(), zone.records)

    def test_apply(self):
        # Create provider with sandbox enabled
        provider = DnsMadeEasyProvider(
            'test', 'api', 'secret', True, strict_supports=False
        )

        resp = Mock()
        resp.json = Mock()
        provider._client._request = Mock(return_value=resp)

        with open('tests/fixtures/dnsmadeeasy-no-domains.json') as fh:
            no_domains = json.load(fh)

        with open('tests/fixtures/dnsmadeeasy-domains.json') as fh:
            domains = json.load(fh)

        with open('tests/fixtures/dnsmadeeasy-domain-create.json') as fh:
            created_domain = json.load(fh)

        # non-existent domain, create everything
        resp.json.side_effect = [
            no_domains,  # no zone in populate
            DnsMadeEasyClientNotFound,  # no domain during apply
            created_domain,  # our created domain response
            domains,
        ]
        plan = provider.plan(self.expected)

        # No ignored, no excluded, no unsupported
        n = len(self.expected.records) - 9
        self.assertEqual(n, len(plan.changes))
        self.assertEqual(n, provider.apply(plan))

        provider._client._request.assert_has_calls(
            [
                # get all domains to build the cache
                call('GET', '/'),
                # attempt to find the domain based on the name
                call('GET', '/id/unit.tests'),
                # create the domain
                call('POST', '/', data={'name': 'unit.tests'}),
                # created all the non-existent records in a single request
                call(
                    'POST',
                    '/123123/records/createMulti',
                    data=[
                        {
                            'value': '1.2.3.4',
                            'name': '',
                            'ttl': 300,
                            'type': 'A',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '1.2.3.5',
                            'name': '',
                            'ttl': 300,
                            'type': 'A',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'aname.unit.tests.',
                            'name': '',
                            'ttl': 1800,
                            'type': 'ANAME',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'ca.unit.tests',
                            'issuerCritical': 0,
                            'name': '',
                            'caaType': 'issue',
                            'ttl': 3600,
                            'type': 'CAA',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'foo-1.unit.tests.',
                            'name': '_srv._tcp',
                            'port': 30,
                            'priority': 10,
                            'ttl': 600,
                            'type': 'SRV',
                            'weight': 20,
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'foo-2.unit.tests.',
                            'name': '_srv._tcp',
                            'port': 30,
                            'priority': 12,
                            'ttl': 600,
                            'type': 'SRV',
                            'weight': 20,
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '2601:644:500:e210:62f8:1dff:feb8:947a',
                            'name': 'aaaa',
                            'ttl': 600,
                            'type': 'AAAA',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'unit.tests.',
                            'name': 'cname',
                            'ttl': 300,
                            'type': 'CNAME',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'unit.tests.',
                            'name': 'included',
                            'ttl': 3600,
                            'type': 'CNAME',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'smtp-4.unit.tests.',
                            'name': 'mx',
                            'mxLevel': 10,
                            'ttl': 300,
                            'type': 'MX',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'smtp-2.unit.tests.',
                            'name': 'mx',
                            'mxLevel': 20,
                            'ttl': 300,
                            'type': 'MX',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'smtp-3.unit.tests.',
                            'name': 'mx',
                            'mxLevel': 30,
                            'ttl': 300,
                            'type': 'MX',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'smtp-1.unit.tests.',
                            'name': 'mx',
                            'mxLevel': 40,
                            'ttl': 300,
                            'type': 'MX',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'foo.bar.com.',
                            'name': 'ptr',
                            'ttl': 300,
                            'type': 'PTR',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '"v=spf1 ip4:192.168.0.1/16-all"',
                            'name': 'spf',
                            'ttl': 600,
                            'type': 'SPF',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '"Bah bah black sheep"',
                            'name': 'txt',
                            'ttl': 600,
                            'type': 'TXT',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '"have you any wool."',
                            'name': 'txt',
                            'ttl': 600,
                            'type': 'TXT',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '"v=DKIM1;k=rsa;s=email;h=sha256;p=A/kinda+of/long/string+with+numb3rs"',
                            'name': 'txt',
                            'ttl': 600,
                            'type': 'TXT',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'ns1.unit.tests.',
                            'name': 'under',
                            'ttl': 3600,
                            'type': 'NS',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': 'ns2.unit.tests.',
                            'name': 'under',
                            'ttl': 3600,
                            'type': 'NS',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '2.2.3.6',
                            'name': 'www',
                            'ttl': 300,
                            'type': 'A',
                            'gtdLocation': 'DEFAULT',
                        },
                        {
                            'value': '2.2.3.6',
                            'name': 'www.sub',
                            'ttl': 300,
                            'type': 'A',
                            'gtdLocation': 'DEFAULT',
                        },
                    ],
                ),
            ]
        )
        self.assertEqual(4, provider._client._request.call_count)

        provider._client._request.reset_mock()

        # delete 1 and update 1
        provider._client.records = Mock(
            return_value=[
                {
                    'id': 11189897,
                    'name': 'www',
                    'value': '1.2.3.4',
                    'ttl': 300,
                    'type': 'A',
                },
                {
                    'id': 11189898,
                    'name': 'www',
                    'value': '2.2.3.4',
                    'ttl': 300,
                    'type': 'A',
                },
                {
                    'id': 11189899,
                    'name': 'ttl',
                    'value': '3.2.3.4',
                    'ttl': 600,
                    'type': 'A',
                },
            ]
        )

        # Domain exists, we don't care about return
        resp.json.side_effect = ['{}']

        wanted = Zone('unit.tests.', [])
        wanted.add_record(
            Record.new(
                wanted, 'ttl', {'ttl': 300, 'type': 'A', 'value': '3.2.3.4'}
            )
        )

        plan = provider.plan(wanted)
        self.assertEqual(2, len(plan.changes))
        self.assertEqual(2, provider.apply(plan))

        # recreate for update, and deletes for the 2 parts of the other
        provider._client._request.assert_has_calls(
            [
                call(
                    'POST',
                    '/123123/records/createMulti',
                    data=[
                        {
                            'value': '3.2.3.4',
                            'type': 'A',
                            'name': 'ttl',
                            'ttl': 300,
                            'gtdLocation': 'DEFAULT',
                        }
                    ],
                ),
                call(
                    'DELETE',
                    '/123123/records',
                    params={'ids': [11189897, 11189898, 11189899]},
                ),
            ],
            any_order=True,
        )
        self.assertEqual(3, provider._client._request.call_count)

        # test for just deleting a record, no additions

        provider._client._request.reset_mock()
        provider._client.records = Mock(
            return_value=[
                {
                    'id': 11189897,
                    'name': 'www',
                    'value': '1.2.3.4',
                    'ttl': 300,
                    'type': 'A',
                }
            ]
        )

        # Domain exists, we don't care about return
        resp.json.side_effect = ['{}']

        wanted = Zone('unit.tests.', [])

        plan = provider.plan(wanted)
        self.assertEqual(1, len(plan.changes))
        self.assertEqual(1, provider.apply(plan))

        # recreate for update, and deletes for the 2 parts of the other
        provider._client._request.assert_has_calls(
            [call('DELETE', '/123123/records', params={'ids': [11189897]})],
            any_order=True,
        )
        self.assertEqual(2, provider._client._request.call_count)



from os import environ
from subprocess import CalledProcessError, check_output

from setuptools import find_packages, setup


def descriptions():
    with open('README.md') as fh:
        ret = fh.read()
        first = ret.split('\n', 1)[0].replace('#', '')
        return first, ret


def version():
    version = 'unknown'
    with open('octodns_dnsmadeeasy/__init__.py') as fh:
        for line in fh:
            if line.startswith('__VERSION__'):
                version = line.split("'")[1]
                break

    # pep440 style public & local version numbers
    if environ.get('OCTODNS_RELEASE', False):
        # public
        return version
    try:
        sha = check_output(['git', 'rev-parse', 'HEAD']).decode('utf-8')[:8]
    except (CalledProcessError, FileNotFoundError):
        sha = 'unknown'
    # local
    return f'{version}+{sha}'


description, long_description = descriptions()

tests_require = ('pytest', 'pytest-cov', 'pytest-network', 'requests_mock')

setup(
    author='Ross McFarland',
    author_email='rwmcfa1@gmail.com',
    description=description,
    extras_require={
        'dev': tests_require
        + (
            'black>=22.3.0',
            'build>=0.7.0',
            # >=5.12.0 does not support python 3.7, we still do
            'isort==5.11.5',
            'pyflakes>=2.2.0',
            'readme_renderer[md]>=26.0',
            'twine>=3.4.2',
        ),
        'test': tests_require,
    },
    install_requires=('octodns>=0.9.14', 'requests>=2.27.0'),
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    name='octodns-dnsmadeeasy',
    packages=find_packages(),
    python_requires='>=3.6',
    tests_require=tests_require,
    url='https://github.com/octodns/octodns-dnsmadeeasy',
    version=version(),
)
